//----------------------------------------------------------------------------------
// File:   CSAA.fx
// Author: Peter Young
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

Texture2D   MeshTexture;
Texture2D   FSTexture;

struct VS_IN
{
	float3	pos			: POSITION;
    float4  color       : COLOR0;
};

struct FS_IN
{
    float4  pos			: SV_Position;
    float4  color       : COLOR0;
};

struct VSQUAD_IN
{
	float3	pos			: POSITION;
    float2  tex0        : TEXCOORD0;
};

struct FSQUAD_IN
{
    float4  pos			: SV_Position;
    float2  tex0        : TEXCOORD0;
};

struct VSMESH_IN
{
	float3	pos			: POSITION;
    float3	normal		: NORMAL;
    float2  tex0        : TEXCOORD0;
};

struct FSMESH_IN
{
    float4  pos			: SV_Position;
    float2  tex0        : TEXCOORD0;
};

SamplerState samplerQuad
{
	Filter = MIN_MAG_MIP_POINT;
	AddressU = Clamp;
	AddressV = Clamp;
};

SamplerState samplerAniso
{
	Filter = ANISOTROPIC;
	AddressU = Wrap;
	AddressV = Wrap;
};

cbuffer cb0
{
	float4x4	worldViewProj;
    float       sOffset;
    float       tOffset;
};

RasterizerState RStateBase
{
//	CullMode = BACK;
	CullMode = NONE;
	FillMode = SOLID;
	MultisampleEnable = TRUE;
};

RasterizerState RStateQuad
{
	CullMode = NONE;
//	FillMode = WIREFRAME;
    FillMode = SOLID;
	MultisampleEnable = TRUE;
};

DepthStencilState EnableDepthTestWrite
{
	DepthEnable = TRUE;
	DepthWriteMask = ALL;
};

///////////////////////////////////////////////////////////////////////////////
// General geometry rendering
///////////////////////////////////////////////////////////////////////////////

FS_IN VSMain( VS_IN IN )
{
	FS_IN OUT;

	OUT.pos = mul(float4(IN.pos, 1), worldViewProj);
    OUT.color = IN.color;

	return OUT;
}

float4 FSMain(FS_IN IN) : SV_Target
{
    return IN.color;
}

///////////////////////////////////////////////////////////////////////////////
// Fullscreen quad rendering
///////////////////////////////////////////////////////////////////////////////

FSQUAD_IN VSQuad( VSQUAD_IN IN )
{
	FSQUAD_IN OUT;

    OUT.pos = mul(float4(IN.pos, 1), worldViewProj);
    OUT.tex0 = IN.tex0;

	return OUT;
}

float4 FSQuad(FSQUAD_IN IN) : SV_Target
{
    float4 color = FSTexture.Sample(samplerQuad, float2(IN.tex0.r + sOffset, IN.tex0.g + tOffset));

	return color;
}

///////////////////////////////////////////////////////////////////////////////
// Mesh rendering
///////////////////////////////////////////////////////////////////////////////

FSMESH_IN VSMesh( VSMESH_IN IN )
{
	FSMESH_IN OUT;

    OUT.pos = mul(float4(IN.pos, 1), worldViewProj);
//    OUT.color = IN.color;
    OUT.tex0 = IN.tex0;

	return OUT;
}

float4 FSMesh(FSMESH_IN IN) : SV_Target
{
    float4 color = MeshTexture.Sample(samplerAniso, IN.tex0);// * IN.color;

	return color;
}

///////////////////////////////////////////////////////////////////////////////
// Zoom border rendering
///////////////////////////////////////////////////////////////////////////////

FS_IN VSZoomBorder( VS_IN IN )
{
	FS_IN OUT;

    IN.pos.x += sOffset;
    IN.pos.y -= tOffset;
	OUT.pos = mul(float4(IN.pos, 1), worldViewProj);

	return OUT;
}

float4 FSZoomBorder(FS_IN IN) : SV_Target
{
	float4 color = float4(1, 0.75, 0, 1);

	return color;
}

///////////////////////////////////////////////////////////////////////////////
// Techniques
///////////////////////////////////////////////////////////////////////////////

technique10 BaseRender
{
	pass p0
	{
		SetVertexShader(CompileShader(vs_4_0, VSMain()));
		SetGeometryShader(NULL);
		SetPixelShader(CompileShader(ps_4_0, FSMain()));

		SetRasterizerState(RStateBase);
		SetDepthStencilState(EnableDepthTestWrite, 0);
	}  
}

technique10 QuadRender
{
    pass p1
    {
        SetVertexShader(CompileShader(vs_4_0, VSQuad()));
		SetGeometryShader(NULL);
		SetPixelShader(CompileShader(ps_4_0, FSQuad()));

		SetRasterizerState(RStateQuad);
		SetDepthStencilState(EnableDepthTestWrite, 0);
    }
}

technique10 MeshRender
{
    pass p1
    {
        SetVertexShader(CompileShader(vs_4_0, VSMesh()));
		SetGeometryShader(NULL);
		SetPixelShader(CompileShader(ps_4_0, FSMesh()));

		SetRasterizerState(RStateQuad);
		SetDepthStencilState(EnableDepthTestWrite, 0);
    }
}

technique10 ZoomBorderRender
{
    pass p1
    {
        SetVertexShader(CompileShader(vs_4_0, VSZoomBorder()));
		SetGeometryShader(NULL);
		SetPixelShader(CompileShader(ps_4_0, FSZoomBorder()));

		SetRasterizerState(RStateQuad);
		SetDepthStencilState(EnableDepthTestWrite, 0);
    }
}
