//----------------------------------------------------------------------------------
// File:   CSAATutorial.cpp
// Author: Peter Young
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#include "DXUT.h"
#include "DXUTgui.h"
#include "DXUTsettingsdlg.h"
#include "DXUTcamera.h"
#include "SDKmisc.h"
#include "sdkmesh_old.h"
#include "SDKmesh.h"
#include "wchar.h"

enum
{
	IDC_TOGGLEFULLSCREEN,
	IDC_TOGGLEREF,
	IDC_CHANGEDEVICE,
    IDC_MESHLOAD,
	IDC_NO_AA,
	IDC_4X,
	IDC_8X,
	IDC_8XQ,
	IDC_16X,
	IDC_16XQ,
    IDC_ROT_TOGGLE,
    IDC_PINWHEEL_TOGGLE,
	IDC_LAST
};

CModelViewerCamera				g_Camera;               // A model viewing camera
CDXUTDialogResourceManager		g_DialogResourceManager; // manager for shared resources of dialogs
CD3DSettingsDlg					g_D3DSettingsDlg;       // Device settings dialog
CDXUTDialog						g_HUD;                  // manages the 3D UI
CDXUTDialog						g_SampleUI;             // dialog for sample specific controls
ID3DX10Font*					g_pFont10 = NULL;       
ID3DX10Sprite*					g_pSprite10 = NULL;

D3DXMATRIX						g_WorldView, g_Projection;

ID3D10Buffer					*g_pVBBasePos, *g_pVBBaseIndex, *g_pVBBaseColor, *g_pVBZoomBorderPos;
ID3D10Buffer					*g_pVBQuadPos, *g_pVBQuadST0, *g_pVBQuadIndex, *g_pVBQuadBorderIndex;
ID3D10Effect					*g_pEffect;
ID3D10EffectTechnique			*g_pTechniqueBase;
ID3D10EffectTechnique			*g_pTechniqueQuad;
ID3D10EffectTechnique			*g_pTechniqueZoomBorder;
ID3D10EffectTechnique			*g_pTechniqueMesh;
ID3D10EffectMatrixVariable		*g_pWorldViewProj = NULL;
ID3D10EffectScalarVariable      *g_pSOffset;
ID3D10EffectScalarVariable      *g_pTOffset;
ID3D10Blob						*g_pBlob;
ID3D10InputLayout				*g_pInputLayoutTri;
ID3D10InputLayout				*g_pInputLayoutQuad;
ID3D10InputLayout				*g_pInputLayoutMesh;

ID3D10Texture2D                     *g_pFSTexture;
ID3D10Texture2D                     *g_pBackbuffer;
ID3D10ShaderResourceView            *g_pFSSRV;
ID3D10EffectShaderResourceVariable	*g_pSRVarTexture;

CDXUTMesh10                         g_Mesh;
WCHAR                               g_FileName[MAX_PATH];

UINT							g_SampleCount = 1;
UINT							g_QualityLevel = 0;

bool							g_bModeSupported = true;
bool                            g_bRot = false;
bool							g_bShowHelp = false;
bool                            g_bPinwheelRender = true;

float                           g_SOffset = 0.54589844f, g_TOffset = 0.18750000f;

void    CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext );
void    RenderText();

// global data
struct Position
{
	float	x;
	float	y;
	float	z;
};

short	g_Indices[] =
{
	0, 1, 2,
};

Position	g_Positions[] =
{
	{ 1.7364f, 10.0f, 0.0f, },
	{ 0.0f, 0.0f, 0.0f, },
	{ -1.7364f, 10.0f, 0.0f, },
};

const float ZOOM_SIZE = 0.05f;
const float ZOOM_SIZE_HALF = ZOOM_SIZE / 2.0f;

float   g_XPos = 0.5f;
float   g_YPos = -0.5f;
float   g_XSize = 0.5f;
float   g_YSize = 0.5f;

Position g_QuadPos[] =
{
    { g_XPos, g_YPos, 0.0f, },
    { g_XPos, g_YPos - g_YSize, 0.0f, },
    { g_XPos + g_XSize, g_YPos - g_YSize, 0.0f, },
    { g_XPos + g_XSize, g_YPos, 0.0f, },
};

Position g_ZoomBorderPos[] =
{
    { -ZOOM_SIZE, -ZOOM_SIZE, 0.0f, },
    { -ZOOM_SIZE, ZOOM_SIZE, 0.0f, },
    { ZOOM_SIZE, ZOOM_SIZE, 0.0f, },
    { ZOOM_SIZE, -ZOOM_SIZE, 0.0f, },
};

float   g_QuadST0[] =
{
    0.0f, 0.0f,
    0.0f, ZOOM_SIZE,
    ZOOM_SIZE, ZOOM_SIZE,
    ZOOM_SIZE, 0.0f,
};

short	g_QuadIndices[] =
{
	0, 1, 3,
    3, 1, 2,
};

short	g_QuadBorderIndices[] =
{
	0, 1, 2, 3, 0,
};

const int   PINWHEEL_DIVISIONS = 32;
const float PINWHEEL_SCALE = 10.0f;

D3DXVECTOR3 *g_pPinwheelPos;
D3DXVECTOR4 *g_pPinwheelColor;
short       *g_pPinwheelIndices;

//--------------------------------------------------------------------------------------
// Callback called when the load model button is pressed.
//--------------------------------------------------------------------------------------
void OnLoadModel()
{
    HRESULT hr;

    OPENFILENAME info;
    ZeroMemory(&info, sizeof (OPENFILENAME));
    info.lStructSize = sizeof (OPENFILENAME);
    info.hwndOwner = DXUTGetHWND();
    info.lpstrFilter = L"X file meshes (.x)\0*.x\0\0";
    info.nFilterIndex = 0;
    info.lpstrFile = g_FileName;
    info.nMaxFile = 256;
    info.lpstrTitle = L"Open model";
    info.nMaxFileTitle = sizeof(info.lpstrTitle);
    info.dwReserved = OFN_EXPLORER;
    info.Flags = OFN_NOCHANGEDIR;

    if (GetOpenFileName(&info))
    {
        const D3D10_INPUT_ELEMENT_DESC inputElements[] =
        {
            { "POSITION",    0, DXGI_FORMAT_R32G32B32_FLOAT,    0, 0,  D3D10_INPUT_PER_VERTEX_DATA, 0 },
            { "NORMAL",      0, DXGI_FORMAT_R32G32B32_FLOAT,    0, 12, D3D10_INPUT_PER_VERTEX_DATA, 0 },
            { "TEXCOORD",    0, DXGI_FORMAT_R32G32_FLOAT,       0, 24, D3D10_INPUT_PER_VERTEX_DATA, 0 },
        };

        UINT numElementsScene = sizeof(inputElements)/sizeof(inputElements[0]);

        hr = g_Mesh.Create(DXUTGetD3D10Device(), g_FileName, (D3D10_INPUT_ELEMENT_DESC*)inputElements, numElementsScene, true);
        if (FAILED(hr))
        {
            MessageBox(NULL, L"Could not load file", NULL, MB_ICONSTOP);
        }
        else
        {
            if (g_bPinwheelRender)
                g_bPinwheelRender = !g_bPinwheelRender;
        }
    }
}

//--------------------------------------------------------------------------------------
// Reject any D3D10 devices that aren't acceptable by returning false
//--------------------------------------------------------------------------------------
bool CALLBACK IsD3D10DeviceAcceptable( UINT Adapter, UINT Output, D3D10_DRIVER_TYPE DeviceType, DXGI_FORMAT BackBufferFormat, bool bWindowed, void* pUserContext )
{
    return true;
}

//--------------------------------------------------------------------------------------
// Initialize the app 
//--------------------------------------------------------------------------------------
HRESULT InitApp()
{
    HRESULT hr;

    g_D3DSettingsDlg.Init( &g_DialogResourceManager );
    g_HUD.Init( &g_DialogResourceManager );
    g_SampleUI.Init( &g_DialogResourceManager );

    g_HUD.SetCallback( OnGUIEvent );
    
    int iY = 10;
    g_HUD.AddButton(IDC_TOGGLEFULLSCREEN, L"Toggle full screen", 35, iY, 125, 22);
    g_HUD.AddButton(IDC_TOGGLEREF, L"Toggle REF (F3)", 35, iY += 24, 125, 22, VK_F3 );
    g_HUD.AddButton(IDC_CHANGEDEVICE, L"Change device (F2)", 35, iY += 24, 125, 22, VK_F2 );
    g_HUD.AddButton(IDC_MESHLOAD, L"Load Mesh (F5)", 35, iY += 24, 125, 22, VK_F5 );
    g_HUD.AddButton(IDC_PINWHEEL_TOGGLE, L"Toggle Pinwheel/Mesh", 35, iY += 24, 125, 22, VK_F6);

    iY += 24;

	g_HUD.AddRadioButton( IDC_NO_AA, 1, L"No AA (S = 1 Q = 0)", 5, iY += 24, 200, 22, true);
	g_HUD.AddRadioButton( IDC_4X, 1, L"4X MSAA (S = 4 Q = 4)", 5, iY += 24, 200, 22, false);
	g_HUD.AddRadioButton( IDC_8X, 1, L"8X CSAA (S = 4 Q = 8)", 5, iY += 24, 200, 22, false);
	g_HUD.AddRadioButton( IDC_8XQ, 1, L"8XQ CSAA (S = 8 Q = 8)", 5, iY += 24, 200, 22, false);
	g_HUD.AddRadioButton( IDC_16X, 1, L"16X CSAA (S = 4 Q = 16)", 5, iY += 24, 200, 22, false);
	g_HUD.AddRadioButton( IDC_16XQ, 1, L"16XQ CSAA (S = 8 Q = 16)", 5, iY += 24, 200, 22, false);

    iY += 24;

    g_HUD.AddCheckBox(IDC_ROT_TOGGLE, L"Toggle Rotation", 5, iY += 24, 200, 22, false);

    g_SampleUI.SetCallback( OnGUIEvent ); iY = 10;

	D3DXVECTOR3	vecEye(0.0f, 0.0f, -15.0f);
    D3DXVECTOR3	vecAt(0, 0, 0);
    g_Camera.SetViewParams(&vecEye, &vecAt);

    V_RETURN(DXUTFindDXSDKMediaFileCch(g_FileName, MAX_PATH, L"..\\Media\\tree.x"));
  
    // build pinwheel
    g_pPinwheelPos = (D3DXVECTOR3 *)malloc(sizeof(D3DXVECTOR3) * PINWHEEL_DIVISIONS * 3);
    g_pPinwheelColor = (D3DXVECTOR4 *)malloc(sizeof(D3DXVECTOR4) * PINWHEEL_DIVISIONS * 3);
    g_pPinwheelIndices = (short *)malloc(sizeof(short) * PINWHEEL_DIVISIONS * 3);

    for (int i = 0, j = 0; i < PINWHEEL_DIVISIONS; i++, j += 3)
    {
        float   angle0 = (float)(i) / (float)PINWHEEL_DIVISIONS * (D3DX_PI * 2);
        float   angle1 = (float)(i + 1) / (float)PINWHEEL_DIVISIONS * (D3DX_PI * 2);

        g_pPinwheelIndices[j] = (short)j;
        g_pPinwheelPos[j] = D3DXVECTOR3(0, 0, 0);
        g_pPinwheelColor[j] = D3DXVECTOR4((float)(i & 1), (float)(i & 1), (float)(i & 1), 1.0f);

        g_pPinwheelIndices[j + 1] = (short)j + 1;
        g_pPinwheelPos[j + 1] = D3DXVECTOR3(cosf(angle0) * PINWHEEL_SCALE, sinf(angle0) * PINWHEEL_SCALE, 0);
        g_pPinwheelColor[j + 1] = D3DXVECTOR4((float)(i & 1), (float)(i & 1), (float)(i & 1), 1.0f);

        g_pPinwheelIndices[j + 2] = (short)j + 2;
        g_pPinwheelPos[j + 2] = D3DXVECTOR3(cosf(angle1) * PINWHEEL_SCALE, sinf(angle1) * PINWHEEL_SCALE, 0);
        g_pPinwheelColor[j + 2] = D3DXVECTOR4((float)(i & 1), (float)(i & 1), (float)(i & 1), 1.0f);
    }

    return hr;
}

//--------------------------------------------------------------------------------------
// Called right before creating a D3D9 or D3D10 device, allowing the app to modify the device settings as needed
//--------------------------------------------------------------------------------------
bool CALLBACK ModifyDeviceSettings( DXUTDeviceSettings* pDeviceSettings, void* pUserContext )
{
	UINT	qualityLevels = 0;

	if (DXUTGetD3D10Device())
	{
		DXUTGetD3D10Device()->CheckMultisampleQualityLevels(pDeviceSettings->d3d10.sd.BufferDesc.Format,
																g_SampleCount,
																&qualityLevels);
	}

	// if mode is not supported, drop to next supported level for this number of color samples
    if (g_SampleCount > 1)
    {
        // remember that CheckMultisampleQualityLevels returns the NUMBER of quality levels
        // so when 16x CSAA is supported, the number of quality levels returned will be 17 as
        // 0-16 are all valid values
	    if ((g_QualityLevel + 1) > qualityLevels)
	    {
		    g_bModeSupported = false;
            g_QualityLevel = qualityLevels == 0 ? 0 : qualityLevels - 1;
	    }
	    else
		    g_bModeSupported = true;
    }
    else
    {
        // failing with 1 sample means we have bigger problems
	    g_bModeSupported = true;
    }

	pDeviceSettings->d3d10.sd.SampleDesc.Count = g_SampleCount;
	pDeviceSettings->d3d10.sd.SampleDesc.Quality = g_QualityLevel;

    return true;
}


//--------------------------------------------------------------------------------------
// Create any D3D10 resources that aren't dependant on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10CreateDevice( ID3D10Device* pd3dDevice, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
	HRESULT hr;

	V_RETURN( g_DialogResourceManager.OnD3D10CreateDevice( pd3dDevice ) );
	V_RETURN( g_D3DSettingsDlg.OnD3D10CreateDevice( pd3dDevice ) );
	V_RETURN( D3DX10CreateFont( pd3dDevice, 15, 0, FW_BOLD, 1, FALSE, DEFAULT_CHARSET, 
								OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, 
								L"Arial", &g_pFont10 ) );
	V_RETURN( D3DX10CreateSprite( pd3dDevice, 512, &g_pSprite10 ) );

	WCHAR str[MAX_PATH];
	if ((hr = DXUTFindDXSDKMediaFileCch(str, MAX_PATH, L"CSAA.fx")) != S_OK)
    {
        MessageBox(DXUTGetHWND(), L"Unable to locate file:  CSAA.fx", L"NVIDIA SDK10 Error", 0);
    }

//	V_RETURN(D3DX10CreateEffectFromFile(str, NULL, NULL, D3D10_SHADER_ENABLE_STRICTNESS, 0, pd3dDevice, NULL, NULL, &g_pEffect, &g_pBlob));
    D3DX10CreateEffectFromFile(str, NULL, NULL, "fx_4_0", D3D10_SHADER_ENABLE_STRICTNESS, 0, pd3dDevice, NULL, NULL, &g_pEffect, &g_pBlob, &hr);

	char	*pData;
	if (g_pBlob)
		pData = (char *)g_pBlob->GetBufferPointer();

	g_pTechniqueBase = g_pEffect->GetTechniqueByName("BaseRender");
    g_pTechniqueQuad = g_pEffect->GetTechniqueByName("QuadRender");
    g_pTechniqueZoomBorder = g_pEffect->GetTechniqueByName("ZoomBorderRender");
    g_pTechniqueMesh = g_pEffect->GetTechniqueByName("MeshRender");
	g_pWorldViewProj = g_pEffect->GetVariableByName("worldViewProj")->AsMatrix();
    g_pSOffset = g_pEffect->GetVariableByName("sOffset")->AsScalar();
    g_pTOffset = g_pEffect->GetVariableByName("tOffset")->AsScalar();
    g_pSRVarTexture = g_pEffect->GetVariableByName("FSTexture")->AsShaderResource();

    // triangle rendering
	const D3D10_INPUT_ELEMENT_DESC	inputElements[] =
	{
		{ "POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "COLOR", 0, DXGI_FORMAT_R32G32B32A32_FLOAT, 1, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 },
	};

	D3D10_PASS_DESC	passDesc;
	g_pTechniqueBase->GetPassByIndex(0)->GetDesc(&passDesc);
	V_RETURN(pd3dDevice->CreateInputLayout(inputElements, 2, passDesc.pIAInputSignature, passDesc.IAInputSignatureSize, &g_pInputLayoutTri));

    // fullscreen quad rendering
    const D3D10_INPUT_ELEMENT_DESC	inputElements2[] =
	{
		{ "POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "TEXCOORD", 0, DXGI_FORMAT_R32G32_FLOAT, 1, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 },
	};

	g_pTechniqueQuad->GetPassByIndex(0)->GetDesc(&passDesc);
	V_RETURN(pd3dDevice->CreateInputLayout(inputElements2, 2, passDesc.pIAInputSignature, passDesc.IAInputSignatureSize, &g_pInputLayoutQuad));

    const D3D10_INPUT_ELEMENT_DESC inputElements3[] =
    {
        { "POSITION",    0, DXGI_FORMAT_R32G32B32_FLOAT,    0, 0,  D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "NORMAL",      0, DXGI_FORMAT_R32G32B32_FLOAT,    0, 12, D3D10_INPUT_PER_VERTEX_DATA, 0 },
        { "TEXCOORD",    0, DXGI_FORMAT_R32G32_FLOAT,       0, 24, D3D10_INPUT_PER_VERTEX_DATA, 0 },
    };

    UINT numElementsScene = sizeof(inputElements3)/sizeof(inputElements3[0]);

    // Create the input layout
    g_pTechniqueMesh->GetPassByIndex(0)->GetDesc(&passDesc);
    V_RETURN(pd3dDevice->CreateInputLayout(inputElements3, numElementsScene, passDesc.pIAInputSignature, passDesc.IAInputSignatureSize, &g_pInputLayoutMesh));
 
    V_RETURN(g_Mesh.Create(pd3dDevice, g_FileName, (D3D10_INPUT_ELEMENT_DESC*)inputElements3, numElementsScene));
   
	// pinwheel position data
    D3D10_BUFFER_DESC	bufDesc;

	bufDesc.ByteWidth = sizeof(D3DXVECTOR3) * PINWHEEL_DIVISIONS * 3;
	bufDesc.Usage = D3D10_USAGE_DEFAULT;
	bufDesc.BindFlags = D3D10_BIND_VERTEX_BUFFER;
	bufDesc.CPUAccessFlags = 0;
	bufDesc.MiscFlags = 0;

	D3D10_SUBRESOURCE_DATA	data;

	data.pSysMem = g_pPinwheelPos;
	data.SysMemPitch = 0;

	V_RETURN(pd3dDevice->CreateBuffer(&bufDesc, &data, &g_pVBBasePos));

    // pinwheel color
    bufDesc.ByteWidth = sizeof(D3DXVECTOR4) * PINWHEEL_DIVISIONS * 3;
	bufDesc.Usage = D3D10_USAGE_DEFAULT;
	bufDesc.BindFlags = D3D10_BIND_VERTEX_BUFFER;
	bufDesc.CPUAccessFlags = 0;
	bufDesc.MiscFlags = 0;

	data.pSysMem = g_pPinwheelColor;
	data.SysMemPitch = 0;

	V_RETURN(pd3dDevice->CreateBuffer(&bufDesc, &data, &g_pVBBaseColor));

	// pinwheel indices
    bufDesc.ByteWidth = sizeof(short) * PINWHEEL_DIVISIONS * 3;
	bufDesc.Usage = D3D10_USAGE_DEFAULT;
	bufDesc.BindFlags = D3D10_BIND_INDEX_BUFFER;
	bufDesc.CPUAccessFlags = 0;
	bufDesc.MiscFlags = 0;

	data.pSysMem = g_pPinwheelIndices;
	data.SysMemPitch = 0;

	V_RETURN(pd3dDevice->CreateBuffer(&bufDesc, &data, &g_pVBBaseIndex));

    // quad position data
	bufDesc.ByteWidth = sizeof(g_QuadPos);
	bufDesc.Usage = D3D10_USAGE_DEFAULT;
	bufDesc.BindFlags = D3D10_BIND_VERTEX_BUFFER;
	bufDesc.CPUAccessFlags = 0;
	bufDesc.MiscFlags = 0;

	data.pSysMem = g_QuadPos;
	data.SysMemPitch = 0;

	V_RETURN(pd3dDevice->CreateBuffer(&bufDesc, &data, &g_pVBQuadPos));

    // quad st coord data
	bufDesc.ByteWidth = sizeof(g_QuadST0);
	bufDesc.Usage = D3D10_USAGE_DEFAULT;
	bufDesc.BindFlags = D3D10_BIND_VERTEX_BUFFER;
	bufDesc.CPUAccessFlags = 0;
	bufDesc.MiscFlags = 0;

	data.pSysMem = g_QuadST0;
	data.SysMemPitch = 0;

	V_RETURN(pd3dDevice->CreateBuffer(&bufDesc, &data, &g_pVBQuadST0));

	// quad indices
	bufDesc.ByteWidth = sizeof(g_QuadIndices);
	bufDesc.Usage = D3D10_USAGE_DEFAULT;
	bufDesc.BindFlags = D3D10_BIND_INDEX_BUFFER;
	bufDesc.CPUAccessFlags = 0;
	bufDesc.MiscFlags = 0;

	data.pSysMem = g_QuadIndices;
	data.SysMemPitch = 0;

	V_RETURN(pd3dDevice->CreateBuffer(&bufDesc, &data, &g_pVBQuadIndex));

    // quad border indices
	bufDesc.ByteWidth = sizeof(g_QuadBorderIndices);
	bufDesc.Usage = D3D10_USAGE_DEFAULT;
	bufDesc.BindFlags = D3D10_BIND_INDEX_BUFFER;
	bufDesc.CPUAccessFlags = 0;
	bufDesc.MiscFlags = 0;

	data.pSysMem = g_QuadBorderIndices;
	data.SysMemPitch = 0;

	V_RETURN(pd3dDevice->CreateBuffer(&bufDesc, &data, &g_pVBQuadBorderIndex));

    // zoom border position data
	bufDesc.ByteWidth = sizeof(g_ZoomBorderPos);
	bufDesc.Usage = D3D10_USAGE_DEFAULT;
	bufDesc.BindFlags = D3D10_BIND_VERTEX_BUFFER;
	bufDesc.CPUAccessFlags = 0;
	bufDesc.MiscFlags = 0;

	data.pSysMem = g_ZoomBorderPos;
	data.SysMemPitch = 0;

	V_RETURN(pd3dDevice->CreateBuffer(&bufDesc, &data, &g_pVBZoomBorderPos));

    return S_OK;
}


//--------------------------------------------------------------------------------------
// Create any D3D10 resources that depend on the back buffer
//--------------------------------------------------------------------------------------
HRESULT CALLBACK OnD3D10ResizedSwapChain( ID3D10Device* pd3dDevice, IDXGISwapChain *pSwapChain, const DXGI_SURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
	HRESULT hr = S_OK;

    // cache backbuffer pointer for AA resolve
    pSwapChain->GetBuffer(0, __uuidof(g_pBackbuffer), (LPVOID *)(&g_pBackbuffer));

	V_RETURN( g_DialogResourceManager.OnD3D10ResizedSwapChain( pd3dDevice, pBackBufferSurfaceDesc ) );

	// Setup the camera's projection parameters
	float fAspectRatio = pBackBufferSurfaceDesc->Width / (FLOAT)pBackBufferSurfaceDesc->Height;
	g_Camera.SetProjParams( D3DX_PI / 2, fAspectRatio, 0.1f, 5000.0f );
	g_Camera.SetWindow( pBackBufferSurfaceDesc->Width, pBackBufferSurfaceDesc->Height );
	g_Camera.SetButtonMasks( MOUSE_LEFT_BUTTON, MOUSE_WHEEL, MOUSE_MIDDLE_BUTTON );

	g_HUD.SetLocation( pBackBufferSurfaceDesc->Width - 170, 0 );
	g_HUD.SetSize( 170, 170 );
	g_SampleUI.SetLocation( pBackBufferSurfaceDesc->Width - 170, pBackBufferSurfaceDesc->Height - 300 );
	g_SampleUI.SetSize( 170, 300 );

    // texture for zoom window
    D3D10_TEXTURE2D_DESC	tex2DDesc;

	memset(&tex2DDesc, 0, sizeof(tex2DDesc));
	tex2DDesc.ArraySize = 1;
	tex2DDesc.BindFlags = D3D10_BIND_RENDER_TARGET | D3D10_BIND_SHADER_RESOURCE;
	tex2DDesc.Format = pBackBufferSurfaceDesc->Format;
    tex2DDesc.Width = pBackBufferSurfaceDesc->Width;
    tex2DDesc.Height = pBackBufferSurfaceDesc->Height;
	tex2DDesc.SampleDesc.Count = 1;
	tex2DDesc.MipLevels = 1;

	V_RETURN(pd3dDevice->CreateTexture2D(&tex2DDesc, NULL, &g_pFSTexture));

    // shader resource view for zoom window
    D3D10_SHADER_RESOURCE_VIEW_DESC	srvd;
	memset(&srvd, 0, sizeof(srvd));
	srvd.Format = pBackBufferSurfaceDesc->Format;
    srvd.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
	srvd.Texture2DArray.MipLevels = 1;
	srvd.Texture2DArray.ArraySize = 1;
	srvd.Texture2DArray.FirstArraySlice = 0;
	srvd.Texture2DArray.MostDetailedMip = 0;

	V_RETURN(pd3dDevice->CreateShaderResourceView(g_pFSTexture, &srvd, &g_pFSSRV));

	return hr;
}


//--------------------------------------------------------------------------------------
// Handle updates to the scene.  This is called regardless of which D3D API is used
//--------------------------------------------------------------------------------------
void CALLBACK OnFrameMove( double fTime, float fElapsedTime, void* pUserContext )
{
	// Update the camera's position based on user input 
	g_Camera.FrameMove( fElapsedTime );
}


//--------------------------------------------------------------------------------------
// Render the scene using the D3D10 device
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10FrameRender( ID3D10Device* pd3dDevice, double fTime, float fElapsedTime, void* pUserContext )
{
	static float	rot = 0;

    if (g_bRot)
        rot += 0.0001f;

	ID3D10RenderTargetView	*pRTView = DXUTGetD3D10RenderTargetView();
	ID3D10DepthStencilView	*pDSView = DXUTGetD3D10DepthStencilView();

	D3D10_RENDER_TARGET_VIEW_DESC	rtvDesc;
	pRTView->GetDesc(&rtvDesc);

    // Clear the depth stencil
    pd3dDevice->ClearDepthStencilView( pDSView, D3D10_CLEAR_DEPTH, 1.0, 0);

	float	clearColor[] = { 0, 0, 0.85f, 0 };
	pd3dDevice->ClearRenderTargetView(pRTView, clearColor);

    // If the settings dialog is being shown, then render it instead of rendering the app's scene
    if( g_D3DSettingsDlg.IsActive() )
    {
        g_D3DSettingsDlg.OnRender( fElapsedTime );
        return;
    }

    // Get the projection & view matrix from the camera class
    D3DXMATRIX world, view, proj, worldViewProj, shit;
	D3DXMatrixRotationZ(&world, rot);
    proj = *g_Camera.GetProjMatrix();
    view = *g_Camera.GetViewMatrix();
	worldViewProj = world * view * proj;
	g_pWorldViewProj->SetMatrix((float *)&worldViewProj);
    g_pSOffset->SetFloat(0);
    g_pTOffset->SetFloat(0);

    // render scene
    pd3dDevice->IASetPrimitiveTopology(D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST);

	UINT	stridesPos[] = { sizeof(D3DXVECTOR3) };
    UINT	stridesColor[] = { sizeof(D3DXVECTOR4) };
    UINT	stridesTex[] = { sizeof(D3DXVECTOR2) };
	UINT	offsets[] = { 0 };

    if (g_bPinwheelRender)
    {
	    pd3dDevice->IASetInputLayout(g_pInputLayoutTri);
        pd3dDevice->IASetVertexBuffers(0, 1, &g_pVBBasePos, stridesPos, offsets);
        pd3dDevice->IASetVertexBuffers(1, 1, &g_pVBBaseColor, stridesColor, offsets);
	    pd3dDevice->IASetIndexBuffer(g_pVBBaseIndex, DXGI_FORMAT_R16_UINT, 0);
        g_pTechniqueBase->GetPassByIndex(0)->Apply(0);
	    pd3dDevice->DrawIndexed(PINWHEEL_DIVISIONS * 3, 0, 0);
    }
    else
    {
        pd3dDevice->IASetInputLayout(g_pInputLayoutMesh);
        g_pTechniqueMesh->GetPassByIndex(0)->Apply(0);
        g_Mesh.Render(pd3dDevice);
    }

    // resolve and copy result to fs texture for zoom window
    pd3dDevice->IASetPrimitiveTopology(D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
    D3D10_TEXTURE2D_DESC   texDesc;
    g_pBackbuffer->GetDesc(&texDesc);

    pd3dDevice->ResolveSubresource(g_pFSTexture, D3D10CalcSubresource(0, 0, 1),
                                    g_pBackbuffer, D3D10CalcSubresource(0, 0, 1),
                                    texDesc.Format);

    // render borders
    // border for zoom window
    D3DXMatrixIdentity(&worldViewProj);
    g_pWorldViewProj->SetMatrix((float *)&worldViewProj);
    g_pSOffset->SetFloat(0);
    g_pTOffset->SetFloat(0);

    pd3dDevice->IASetPrimitiveTopology(D3D10_PRIMITIVE_TOPOLOGY_LINESTRIP);
    pd3dDevice->IASetInputLayout(g_pInputLayoutTri);
    pd3dDevice->IASetVertexBuffers(0, 1, &g_pVBQuadPos, stridesPos, offsets);
	pd3dDevice->IASetIndexBuffer(g_pVBQuadBorderIndex, DXGI_FORMAT_R16_UINT, 0);
    g_pTechniqueZoomBorder->GetPassByIndex(0)->Apply(0);
	pd3dDevice->DrawIndexed(5, 0, 0);

    // border for zoom window origin
    D3DXMatrixIdentity(&worldViewProj);
    g_pWorldViewProj->SetMatrix((float *)&worldViewProj);
    g_pSOffset->SetFloat((g_SOffset * 2.0f) - 1.0f);
    g_pTOffset->SetFloat((g_TOffset * 2.0f) - 1.0f);

    pd3dDevice->IASetPrimitiveTopology(D3D10_PRIMITIVE_TOPOLOGY_LINESTRIP);
    pd3dDevice->IASetInputLayout(g_pInputLayoutTri);
    pd3dDevice->IASetVertexBuffers(0, 1, &g_pVBZoomBorderPos, stridesPos, offsets);
	pd3dDevice->IASetIndexBuffer(g_pVBQuadBorderIndex, DXGI_FORMAT_R16_UINT, 0);
    g_pTechniqueZoomBorder->GetPassByIndex(0)->Apply(0);
	pd3dDevice->DrawIndexed(5, 0, 0);

    // render zoom window
    pd3dDevice->IASetPrimitiveTopology(D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
    pd3dDevice->IASetInputLayout(g_pInputLayoutQuad);
    D3DXMatrixIdentity(&worldViewProj);
    g_pWorldViewProj->SetMatrix((float *)&worldViewProj);
    g_pSRVarTexture->SetResource(g_pFSSRV);
    g_pSOffset->SetFloat(g_SOffset - ZOOM_SIZE_HALF);
    g_pTOffset->SetFloat(g_TOffset - ZOOM_SIZE_HALF);

    pd3dDevice->IASetVertexBuffers(0, 1, &g_pVBQuadPos, stridesPos, offsets);
    pd3dDevice->IASetVertexBuffers(1, 1, &g_pVBQuadST0, stridesTex, offsets);
	pd3dDevice->IASetIndexBuffer(g_pVBQuadIndex, DXGI_FORMAT_R16_UINT, 0);

    g_pTechniqueQuad->GetPassByIndex(0)->Apply(0);
    pd3dDevice->DrawIndexed(6, 0, 0);

    g_HUD.OnRender(fElapsedTime);
	g_SampleUI.OnRender(fElapsedTime);

    RenderText();
}

//--------------------------------------------------------------------------------------
// Render the help and statistics text
//--------------------------------------------------------------------------------------
void RenderText()
{
    CDXUTTextHelper txtHelper( g_pFont10, g_pSprite10, 15 );
    txtHelper.Begin();
    txtHelper.SetInsertionPos( 2, 0 );
    txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 0.0f, 1.0f ) );
    txtHelper.DrawTextLine( DXUTGetFrameStats(true) );
    txtHelper.DrawTextLine( DXUTGetDeviceStats() );

	if (g_bModeSupported == false)
	{
		txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 0.0f, 0.0f, 1.0f ) );
		txtHelper.DrawTextLine(L"WARNING: Selected CSAA mode is not supported!\n");
		txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 0.0f, 1.0f ) );
	}

    // Draw help
    if( g_bShowHelp )
    {
        UINT nBackBufferHeight = ( DXUTIsAppRenderingWithD3D9() ) ? DXUTGetD3D9BackBufferSurfaceDesc()->Height : DXUTGetDXGIBackBufferSurfaceDesc()->Height;
        txtHelper.SetInsertionPos( 2, nBackBufferHeight-15*7 );
        txtHelper.SetForegroundColor( D3DXCOLOR(1.0f, 0.75f, 0.0f, 1.0f ) );
        txtHelper.DrawTextLine( L"Controls:" );

        txtHelper.SetInsertionPos( 20, nBackBufferHeight-15*6 );
        txtHelper.DrawTextLine( L"Rotate model:  Middle mouse button\n"
                                L"Change zoom origin:  Right mouse button\n"
                                L"Zoom camera:  Mouse wheel scroll\n\n"
                                L"Note:  CSAA settings override the multisample settings in the Device Menu\n");

        txtHelper.SetInsertionPos( 280, nBackBufferHeight-15*6 );
        txtHelper.DrawTextLine( L"Hide help: F1\n" 
                                L"Quit: ESC\n" );
    }
    else
    {
        txtHelper.SetForegroundColor( D3DXCOLOR( 1.0f, 1.0f, 1.0f, 1.0f ) );
        txtHelper.DrawTextLine( L"Press F1 for help" );
    }

    txtHelper.End();
}

//--------------------------------------------------------------------------------------
// Recreate device upon AA mode changes
//--------------------------------------------------------------------------------------
void ChangeAAMode() {
    // if fullscreen, recreating device in fullscreen has error in DXUT, so create it in window mode, and change it back to fullscreen.
    if (DXUTIsWindowed()) 
	{
        DXUTCreateDevice(true, 0, 0);
    }
	else 
	{
        SendMessageW(DXUTGetHWND(), WM_SYSKEYDOWN, VK_RETURN, (1 << 29));
        DXUTCreateDevice(true, 0, 0);
        PostMessageW(DXUTGetHWND(), WM_SYSKEYDOWN, VK_RETURN, (1 << 29));
    } 
}


//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10ResizedSwapChain 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10ReleasingSwapChain( void* pUserContext )
{
	g_DialogResourceManager.OnD3D10ReleasingSwapChain();

    g_pBackbuffer->Release();
    g_pFSTexture->Release();
    g_pFSSRV->Release();
}


//--------------------------------------------------------------------------------------
// Release D3D10 resources created in OnD3D10CreateDevice 
//--------------------------------------------------------------------------------------
void CALLBACK OnD3D10DestroyDevice( void* pUserContext )
{
	g_DialogResourceManager.OnD3D10DestroyDevice();
	g_D3DSettingsDlg.OnD3D10DestroyDevice();

	g_pEffect->Release();

	g_pVBBasePos->Release();
    g_pVBBaseColor->Release();
	g_pVBBaseIndex->Release();
	g_pInputLayoutTri->Release();

    g_pBackbuffer->Release();
    g_pVBQuadPos->Release();
    g_pVBQuadST0->Release();
	g_pVBQuadIndex->Release();
    g_pVBQuadBorderIndex->Release();
	g_pInputLayoutQuad->Release();

    g_pInputLayoutMesh->Release();
    g_Mesh.Destroy();

    g_pVBZoomBorderPos->Release();

	g_pSprite10->Release();
	g_pFont10->Release();
}


//--------------------------------------------------------------------------------------
// Handle messages to the application
//--------------------------------------------------------------------------------------
LRESULT CALLBACK MsgProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, 
                          bool* pbNoFurtherProcessing, void* pUserContext )
{
	// Pass messages to dialog resource manager calls so GUI state is updated correctly
	*pbNoFurtherProcessing = g_DialogResourceManager.MsgProc( hWnd, uMsg, wParam, lParam );
	if( *pbNoFurtherProcessing )
		return 0;

	// Pass messages to settings dialog if its active
	if( g_D3DSettingsDlg.IsActive() )
	{
		g_D3DSettingsDlg.MsgProc( hWnd, uMsg, wParam, lParam );
		return 0;
	}

	// Give the dialogs a chance to handle the message first
	*pbNoFurtherProcessing = g_HUD.MsgProc( hWnd, uMsg, wParam, lParam );
	if( *pbNoFurtherProcessing )
		return 0;
	*pbNoFurtherProcessing = g_SampleUI.MsgProc( hWnd, uMsg, wParam, lParam );
	if( *pbNoFurtherProcessing )
		return 0;

	// Pass all remaining windows messages to camera so it can respond to user input
	g_Camera.HandleMessages( hWnd, uMsg, wParam, lParam );

	return 0;
}


//--------------------------------------------------------------------------------------
// Handle key presses
//--------------------------------------------------------------------------------------
void CALLBACK OnKeyboard( UINT nChar, bool bKeyDown, bool bAltDown, void* pUserContext )
{
    if( bKeyDown )
    {
        switch( nChar )
        {
            case VK_F1: g_bShowHelp = !g_bShowHelp;
            break;
        }
    }
}

//--------------------------------------------------------------------------------------
// Handles the GUI events
//--------------------------------------------------------------------------------------
void CALLBACK OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext )
{
	switch( nControlID )
    {
        case IDC_TOGGLEFULLSCREEN:
            PostMessageW(DXUTGetHWND(), WM_SYSKEYDOWN, VK_RETURN, (1 << 29));
            //DXUTToggleFullScreen();
        break;

        case IDC_TOGGLEREF:
            g_SampleCount = 1;
			g_QualityLevel = 0;
            g_HUD.GetRadioButton(IDC_NO_AA)->SetChecked(true, true);
            DXUTToggleREF();
        break;

        case IDC_CHANGEDEVICE:
            g_D3DSettingsDlg.SetActive( !g_D3DSettingsDlg.IsActive() );
        break;

        case IDC_MESHLOAD:
            OnLoadModel();
        break;

        case IDC_ROT_TOGGLE:
            g_bRot = !g_bRot;
        break;

        case IDC_PINWHEEL_TOGGLE:
            g_bPinwheelRender = !g_bPinwheelRender;
        break;

		// no AA
		case IDC_NO_AA:
			g_SampleCount = 1;
			g_QualityLevel = 0;
            ChangeAAMode();
		break;

		// 4X MSAA
		case IDC_4X:
			g_SampleCount = 4;
			g_QualityLevel = 4;
            ChangeAAMode();
 		break;

		// 8X CSAA
		case IDC_8X:
			g_SampleCount = 4;
			g_QualityLevel = 8;
            ChangeAAMode();
		break;

		// 8XQ CSAA
		case IDC_8XQ:
			g_SampleCount = 8;
			g_QualityLevel = 8;
            ChangeAAMode();
		break;

		// 16X CSAA
		case IDC_16X:
			g_SampleCount = 4;
			g_QualityLevel = 16;
            ChangeAAMode();
		break;

		// 16XQ CSAA
		case IDC_16XQ:
			g_SampleCount = 8;
			g_QualityLevel = 16;
            ChangeAAMode();
		break;
    }
}


//--------------------------------------------------------------------------------------
// Handle mouse button presses
//--------------------------------------------------------------------------------------
void CALLBACK OnMouse( bool bLeftButtonDown, bool bRightButtonDown, bool bMiddleButtonDown, 
                       bool bSideButton1Down, bool bSideButton2Down, int nMouseWheelDelta, 
                       int xPos, int yPos, void* pUserContext )
{
    if (bRightButtonDown)
    {
        RECT    rect;
        rect = DXUTGetWindowClientRect();

        g_SOffset = (float)xPos / (float)rect.right;
        g_TOffset = (float)yPos / (float)rect.bottom;
    }
}


//--------------------------------------------------------------------------------------
// Call if device was removed.  Return true to find a new device, false to quit
//--------------------------------------------------------------------------------------
bool CALLBACK OnDeviceRemoved( void* pUserContext )
{
    return true;
}


//--------------------------------------------------------------------------------------
// Initialize everything and go into a render loop
//--------------------------------------------------------------------------------------
int WINAPI wWinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPWSTR lpCmdLine, int nCmdShow )
{
    // Enable run-time memory check for debug builds.
#if defined(DEBUG) | defined(_DEBUG)
    _CrtSetDbgFlag( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
#endif

    // DXUT will create and use the best device (either D3D9 or D3D10) 
    // that is available on the system depending on which D3D callbacks are set below

    // Set general DXUT callbacks
    DXUTSetCallbackFrameMove( OnFrameMove );
    DXUTSetCallbackKeyboard( OnKeyboard );
    DXUTSetCallbackMouse( OnMouse );
    DXUTSetCallbackMsgProc( MsgProc );
    DXUTSetCallbackDeviceChanging( ModifyDeviceSettings );
    DXUTSetCallbackDeviceRemoved( OnDeviceRemoved );

   // Set the D3D10 DXUT callbacks. Remove these sets if the app doesn't need to support D3D10
    DXUTSetCallbackD3D10DeviceAcceptable( IsD3D10DeviceAcceptable );
    DXUTSetCallbackD3D10DeviceCreated( OnD3D10CreateDevice );
    DXUTSetCallbackD3D10SwapChainResized( OnD3D10ResizedSwapChain );
    DXUTSetCallbackD3D10FrameRender( OnD3D10FrameRender );
    DXUTSetCallbackD3D10SwapChainReleasing( OnD3D10ReleasingSwapChain );
    DXUTSetCallbackD3D10DeviceDestroyed( OnD3D10DestroyDevice );

	// IMPORTANT: set SDK media search path to include source directory of this sample, when started from .\Bin
	HRESULT hr;
	V_RETURN(DXUTSetMediaSearchPath(L"..\\Source\\CSAATutorial"));
	
    // Perform any application-level initialization here

    InitApp();
    DXUTInit( true, true, NULL ); // Parse the command line, show msgboxes on error, no extra command line params
    DXUTSetCursorSettings( true, true ); // Show the cursor and clip it when in full screen
    DXUTCreateWindow( L"CSAA Tutorial" );
    DXUTCreateDevice( true, 1024, 768);
    DXUTMainLoop(); // Enter into the DXUT render loop
 
    // Perform any application-level cleanup here
    free(g_pPinwheelPos);
    free(g_pPinwheelColor);
    free(g_pPinwheelIndices);

    return DXUTGetExitCode();
}


